package com.mcwl.resource.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.WorkFlowComment;
import com.mcwl.resource.domain.vo.WorkFlowCommentVo;
import com.mcwl.resource.mapper.WorkFlowCommentMapper;
import com.mcwl.resource.service.WorkFlowCommentService;
import com.mcwl.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 工作流评论 实现层
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：ModelCommentServiceImpl
 * @Description TODO
 * @Date：2025/1/12 12:03
 */
@Service
public class WorkFlowCommentServiceImpl extends ServiceImpl<WorkFlowCommentMapper, WorkFlowComment> implements WorkFlowCommentService {

    @Autowired
    private  ISysUserService sysUserService;

    @Autowired
    private  WorkFlowCommentMapper workFlowCommentMapper;


    @Override
    public void comment(WorkFlowComment workFlowComment) {
        Long parentId = workFlowComment.getParentId();
        if (parentId != null){
            WorkFlowComment mic = workFlowCommentMapper.selectById(parentId);

            if (Objects.nonNull(parentId) && Objects.isNull(mic)) {
                return;
            }
        }
        workFlowComment.setUserId(SecurityUtils.getUserId());
        workFlowComment.setCreateBy(SecurityUtils.getUsername());
        workFlowComment.setUpdateBy(SecurityUtils.getUsername());
        workFlowComment.setUpdateTime(new Date());
        workFlowCommentMapper.insert(workFlowComment);

    }

    @Override
    public List<WorkFlowCommentVo> getComment(Long imageId) {
        List<WorkFlowCommentVo> workFlowCommentVos = new ArrayList<>();

        // 查询所有父评论
        LambdaQueryWrapper<WorkFlowComment> lqw = new LambdaQueryWrapper<>();
        lqw.eq(WorkFlowComment::getWorkFlowId, imageId)
                .isNull(WorkFlowComment::getParentId)
                .orderByDesc(WorkFlowComment::getCreateTime);
        // 添加父评论
        List<WorkFlowComment> workFlowComments = workFlowCommentMapper.selectList(lqw);
        for (WorkFlowComment workFlowComment : workFlowComments) {
            WorkFlowCommentVo workFlowCommentVo = getModelCommentVo(workFlowComment);
            workFlowCommentVos.add(workFlowCommentVo);
        }

        return workFlowCommentVos;
    }


    /**
     * 构建ModelCommentVo对象
     *
     * @param workFlowComment 父评论对象
     * @return WorkFlowCommentVo
     */
    @NotNull
    private WorkFlowCommentVo getModelCommentVo(WorkFlowComment workFlowComment) {
        Long userId = workFlowComment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建WorkFlowCommentVo对象
        WorkFlowCommentVo workFlowCommentVo = new WorkFlowCommentVo();
        workFlowCommentVo.setUserId(userId);
        workFlowCommentVo.setUserName(sysUser.getUserName());
        workFlowCommentVo.setUserAvatar(sysUser.getAvatar());
        workFlowCommentVo.setCommentId(workFlowComment.getId());
        workFlowCommentVo.setContent(workFlowComment.getContent());
        // 查询子评论
        workFlowCommentVo.setContentList(getContentList(workFlowComment.getId()));
        workFlowCommentVo.setLikeNum(workFlowComment.getLikeNum());
        workFlowCommentVo.setCreateTime(workFlowComment.getCreateTime());
        return workFlowCommentVo;
    }


    /**
     * 递归查询子评论
     *
     * @param modelCommentId 父评论id
     * @return List<ModelCommentVo>
     */
    private List<WorkFlowCommentVo> getContentList(Long modelCommentId) {
        List<WorkFlowCommentVo> modelCommentVoList = new ArrayList<>();
        if (Objects.isNull(modelCommentId)) {
            return modelCommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<WorkFlowComment> lqw = new LambdaQueryWrapper<WorkFlowComment>()
                .eq(WorkFlowComment::getParentId, modelCommentId)
                .orderByDesc(WorkFlowComment::getCreateTime);

        List<WorkFlowComment> modelCommentList = workFlowCommentMapper.selectList(lqw);

        for (WorkFlowComment modelComment : modelCommentList) {
            Long userId = modelComment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            WorkFlowCommentVo modelCommentVo = new WorkFlowCommentVo();
            modelCommentVo.setUserId(userId);
            modelCommentVo.setUserName(sysUser.getUserName());
            modelCommentVo.setUserAvatar(sysUser.getAvatar());
            modelCommentVo.setCommentId(modelComment.getId());
            modelCommentVo.setContent(modelComment.getContent());
            modelCommentVo.setLikeNum(modelComment.getLikeNum());
            modelCommentVo.setCreateTime(modelComment.getCreateTime());


            modelCommentVoList.add(modelCommentVo);
        }
        return modelCommentVoList;
    }


}
